/*
 * BoomTimer application by Denis Volkov (C) 2010
 * Canvas and timer function class
 */

package boomtimer;

import java.util.Random;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.media.Manager;

public class BoomCanvas extends Canvas implements Runnable, CommandListener
{
  /**
   * Number of seconds before "boom"
   */
  private int elapsed = 0;

  /**
   * Animation counter
   */
  private int animationCounter = -1;

  /**
   * Animation images
   */
  private final Image[] clockImg = new Image[5];
  private final Image[] boomImg = new Image[2];

  /**
   * Midlet object
   */
  private final BoomMIDlet midlet;

  /**
   * Thread object
   */
  private final Thread th;

  /**
   * Random object
   */
  private final Random rand;

  /**
   * Commands
   */
  private final Command start;
  private final Command stop;
  private final Command exit;


  private boolean stopFlag = false;

    /**
   * Creates new canvas
   * @param m
   */

  public BoomCanvas(BoomMIDlet m)
  {
    midlet = m;

    // Load images
    try
    {
      clockImg[0] = Image.createImage("/img/clock0.png");
      clockImg[1] = Image.createImage("/img/clock1.png");
      clockImg[2] = Image.createImage("/img/clock2.png");
      clockImg[3] = Image.createImage("/img/clock3.png");
      clockImg[4] = Image.createImage("/img/clock4.png");
      boomImg[0] = Image.createImage("/img/clock0.png");
      boomImg[1] = Image.createImage("/img/clock4.png");
    }
    catch(Exception e) {}

    rand = new Random();

    start = new Command("Start", Command.OK, 1);
    stop = new Command("Stop", Command.CANCEL, 1);
    exit = new Command("Exit", Command.EXIT, 1);

    setCommandListener(this);

    addCommand(start);
    addCommand(exit);

    th = new Thread(this);
    th.start();
  }

  public void startTimer()
  {
    elapsed = 30 + (Math.abs(rand.nextInt()) % 90);
    animationCounter = 0;

    removeCommand(start);
    removeCommand(exit);
    addCommand(stop);
  }

  public void stopTimer()
  {
    elapsed = 0;
    animationCounter = 10;
    removeCommand(stop);
    addCommand(start);
    addCommand(exit);
  }

  public void paint(Graphics g)
  {
    int w2 = getWidth() >> 1;
    int h2 = getHeight() >> 1;
    int r = Math.min(w2, h2);
    int x = w2 - r, y = h2 - r;

    g.setGrayScale(255);
    g.fillRect(0, 0, w2 << 1, h2 << 1);
    if(elapsed > 0)
    {
      g.drawImage(clockImg[animationCounter], w2, h2, Graphics.HCENTER | Graphics.VCENTER);
    }
    else if(animationCounter >= 0)
    {
      if(animationCounter % 2 == 0)
      {
        g.setGrayScale(0);
        g.fillRect(0, 0, w2 << 1, h2 << 1);
      }
    }
  }

  public void run()
  {
    while(!stopFlag)
    {
      if(elapsed > 0)
      {
        ++ animationCounter;
        if(animationCounter >= 5)
        {
          animationCounter = 0;
          -- elapsed;
          if(elapsed == 0)
            stopTimer();
          else
          {
            try
            {
              Manager.playTone(75, 100, 50);
            }
            catch(Exception e) { }
          }
        }
      }
      else if(animationCounter >= 0)
      {
        -- animationCounter;
        try
        {
          Manager.playTone((animationCounter % 2 == 0) ? 69 : 79, 450, 100);
        }
        catch(Exception e) { }

      }

      repaint();

      try
      {
        Thread.sleep(elapsed > 0 ? 200 : 500);
      }
      catch(Exception e) {}
    }
  }

  public void commandAction(Command command, Displayable displayable)
  {
    if(command == start)
      startTimer();
    else if(command == stop)
      stopTimer();
    else if(command == exit)
    {
      stopFlag = true;
      midlet.exitMIDlet();
    }
  }
}
